#!/usr/bin/env bash

#This script will not update number of replicas for a topic
inputfile=$1
/opt/kafka/bin/kafka-topics.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --describe | grep Configs > /opt/kafka/current_topics_list.txt
echo "*****  Current topics List and config  *****"
cat /opt/kafka/current_topics_list.txt
echo "*****  End of List  *****"
new_topic=0
while read line
do
topic='None'
partition='None'
replica='None'
cleanupPolicy='None'
retentionMs='None'
maxMessageBytes='None'
if ! [ -z "$line" ]; then
topic=$(echo $line | awk '{print $1}' | cut -d "=" -f2)
partition=$(echo $line | awk '{print $2}' | cut -d "=" -f2)
replica=$(echo $line | awk '{print $3}' | cut -d '=' -f2)
cleanupPolicy=$(echo $line | awk -F= '$1=="cleanupPolicy"{print $2}' RS=' ' | xargs)
retentionMs=$(echo $line | awk -F= '$1=="retentionMs"{print $2}' RS=' ' | xargs)
maxMessageBytes=$(echo $line | awk -F= '$1=="maxMessageBytes"{print $2}' RS=' ' | xargs)
grep -E "\s+$topic\s+"  /opt/kafka/current_topics_list.txt 2>&1 > /dev/null
if [ $? -ne 0 ]; then
  new_topic=1
  echo Create new $topic
  CMD_PARAM=''
  if ! [ -z "$cleanupPolicy" ]; then
    CMD_PARAM='--config cleanup.policy='$cleanupPolicy' '
  fi
  if ! [ -z "$retentionMs" ]; then
    CMD_PARAM=$CMD_PARAM'--config retention.ms='$retentionMs' '
  fi
  if ! [ -z "$maxMessageBytes" ]; then
    CMD_PARAM=$CMD_PARAM'--config max.message.bytes='$maxMessageBytes' '
  fi
  /opt/kafka/bin/kafka-topics.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --if-not-exists --create --partitions $partition --replication-factor $replica --topic $topic $CMD_PARAM
  /opt/kafka/bin/kafka-topics.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --describe --topic $topic
else
  current_partition=$(grep -E "\s+$topic\s+" /opt/kafka/current_topics_list.txt | awk '{print $6}')
  current_replica=$(grep -E "\s+$topic\s+" /opt/kafka/current_topics_list.txt | awk '{print $8}')
  current_cleanup_policy=$(grep -E "\s+$topic\s+" /opt/kafka/current_topics_list.txt | awk '{print $10}'  | awk -F= '$1=="cleanup.policy"{print $2}' RS=',' | xargs)
  current_retention_ms=$(grep -E "\s+$topic\s+" /opt/kafka/current_topics_list.txt | awk '{print $10}'  | awk -F= '$1=="retention.ms"{print $2}' RS=',' | xargs)
  current_max_message_bytes=$(grep -E "\s+$topic\s+" /opt/kafka/current_topics_list.txt | awk '{print $10}'  | awk -F= '$1=="max.message.bytes"{print $2}' RS=',' | xargs)

  if [[ "$partition" != "$current_partition" && "$partition" -lt "$current_partition" ]]; then
    echo "Cannot decrease the number of partitions for $topic topic. Current partitions are: $current_partition and trying to create $partition partitions."
  fi

  if [[ "$partition" != "$current_partition" && "$partition" -gt "$current_partition" ]]; then
    /opt/kafka/bin/kafka-topics.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --alter --topic $topic --partitions $partition
    echo "Changing the number of partitions for $topic topic from $current_partition to $partition."
  fi

  if ! [ -z "$cleanupPolicy" ]; then
    if [[ "$cleanupPolicy" != "$current_cleanup_policy" ]]; then
      CLEANUP_POLICY="cleanup.policy=$cleanupPolicy"
      /opt/kafka/bin/kafka-configs.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --alter --entity-type topics --entity-name $topic --add-config $CLEANUP_POLICY
      echo "Changing the cleanup policy for $topic topic from $current_cleanup_policy to $cleanupPolicy."
    fi
  fi

  if ! [ -z "$retentionMs" ]; then
    if [[ "$retentionMs" != "$current_retention_ms" ]]; then
      RETENTION_MS="retention.ms="$retentionMs
      /opt/kafka/bin/kafka-configs.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --alter --entity-type topics --entity-name $topic --add-config $RETENTION_MS
      echo "Changing the retention.ms for $topic topic from $current_retention_ms to $retentionMs."
    fi
  fi

  if ! [ -z "$maxMessageBytes" ]; then
    if [[ "$maxMessageBytes" != "$current_max_message_bytes" ]]; then
      MAX_MESSAGE_BYTES="max.message.bytes="$maxMessageBytes
      /opt/kafka/bin/kafka-configs.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --alter --entity-type topics --entity-name $topic --add-config $MAX_MESSAGE_BYTES
      echo "Changing the max.message.bytes for $topic topic from $current_max_message_bytes to $maxMessageBytes."
    fi
  fi
fi
fi
done <  $inputfile
if [ "$new_topic" -eq 0 ]; then
  echo No new topics to create
fi
echo -e "\n\n"Current exiting topics:
/opt/kafka/bin/kafka-topics.sh --bootstrap-server localhost:9092 --command-config /opt/kafka/consumer.properties --list

rm /opt/kafka/current_topics_list.txt